@@ -1,7 +1,8 @@
 use std::borrow::Borrow;
 use std::fmt;
 use std::hash::{Hash, Hasher};
-use std::ops::Deref;
+use std::mem;
+use std::ops::{Deref, DerefMut};
 use std::str;
 use std::slice;
 
@@ -99,7 +100,7 @@ impl<A: Array<Item=u8>> ArrayString<A> {
     pub fn push_str<'a>(&mut self, s: &'a str) -> Result<(), CapacityError<&'a str>> {
         use std::io::Write;
 
-        if self.len() + s.len() > self.capacity() {
+        if s.len() > self.capacity() - self.len() {
             return Err(CapacityError::new(s));
         }
         unsafe {
@@ -142,6 +143,17 @@ impl<A: Array<Item=u8>> Deref for ArrayString<A> {
     }
 }
 
+impl<A: Array<Item=u8>> DerefMut for ArrayString<A> {
+    #[inline]
+    fn deref_mut(&mut self) -> &mut str {
+        unsafe {
+            let sl = slice::from_raw_parts_mut(self.xs.as_mut_ptr(), self.len.to_usize());
+            // FIXME: Nothing but transmute to do this right now
+            mem::transmute(sl)
+        }
+    }
+}
+
 impl<A: Array<Item=u8>> PartialEq for ArrayString<A> {
     fn eq(&self, rhs: &Self) -> bool {
         **self == **rhs
